/* AlGnomegebraic
 * Copyright (C) 1999 the Free Software Foundation
 * Authors : Matias Mutchinick          
 *         
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include "ag_defs.h"
#include "errorstring.h"
#include "evalstring.h"
#include "formatstring.h"


extern AgError errState;

void 
str_result_of_expr(gchar      *exprStr, 
		   gchar      *resultStr,
		   gchar      *numberFormatStr, 
		   gchar      *numberFormatSizeStr,
		   gchar      *angleMessureStr)
{
  
	gdouble numericResult;
	gint angleMessure = 0,numberFormatSize = 0, numberFormat = 0;
	
		
	if(!strcmp(angleMessureStr,"Rad "))
		angleMessure = RAD;
	else if(!strcmp(angleMessureStr,"Grad"))
		angleMessure = GRAD;
	else if(!strcmp(angleMessureStr,"Deg "))
		angleMessure = DEG;
	
	
	if(!strcmp(numberFormatStr,"Std"))
		numberFormat = STD;
	else if(!strcmp(numberFormatStr,"Fix"))
		numberFormat = FIX;
	else if(!strcmp(numberFormatStr,"Eng"))
		numberFormat = ENG;
	else if(!strcmp(numberFormatStr,"Sci"))
		numberFormat = SCI;
	
	
	numberFormatSize = atoi(numberFormatSizeStr);
	
	errState.state = 0; /* Clear error */
	
	numericResult = 
		eval_string(exprStr,numberFormat,
			    numberFormatSize,angleMessure);
	
	if(errState.state) {
		str_error(resultStr);
		return;
	}
	
	switch(numberFormat){
	case FIX: /* case Fix */
		str_fix_format(numericResult,numberFormatSize,resultStr);
		break;    
		
	case ENG: /* Case Eng */
		str_eng_format(numericResult,numberFormatSize,resultStr);
		break;
		
	case SCI: /* Case S.. */
		str_sci_format(numericResult,numberFormatSize,resultStr);
		break;
		
	case STD: /* Case Std */
		gcvt(numericResult,numberFormatSize,resultStr);
		break;
	}
	
	return;
}


/************************************************************************/
/* Funcion: str_fix_format                                              */
/* Entrada: cadena, double, numero de digitos                           */
/* Salida : convierte un double a una cadena formato FIX                */
/************************************************************************/ 

void 
str_fix_format(gdouble       numericResult, 
	       gint          formatSize,
	       gchar        *resultStr)
{
  gint    dec, sign, expSize = 0, expSign = 0;
  gchar  *tmpStr;
  



  tmpStr = ecvt(numericResult,MAX_FORMAT_SIZE,&dec,&sign);

  
  if( dec > MAX_FORMAT_SIZE ) { 
    expSize = dec - 1;
    dec = 1;
  } 
  else 
    if ( dec <= -formatSize ) {
      expSize = 1 - dec;
      dec = 1;
      expSign = 1;
    }
  
  if( dec + formatSize < MAX_FORMAT_SIZE )
    *(tmpStr + dec + formatSize) = '\0';
  
  if(sign) 
    *resultStr++ = '-';
  
  if(dec <= 0 ) {
    *resultStr++ = '0'; 
    *resultStr++ = '.';
    while(dec < 0) {
      *resultStr++ = '0'; 
      dec++;
    }
    while( *tmpStr != '\0' )
      *resultStr++ = *tmpStr++;
  }
  else {
    while( *tmpStr != '\0' ) {
      if( dec == 0 ) 
	*resultStr++ = '.';
      *resultStr++ = *tmpStr++;
      dec--;
    }
  }
  
  if(expSize != 0) { 
    *resultStr++ = 'e';
    if(expSign)
      *resultStr++ = '-';
    else
      *resultStr++ = '+';
    gcvt(expSize,4,resultStr);
  } 
  else 
    *resultStr = '\0'; 
  
  return;
}


/************************************************************************/
/* Funcion: str_sci_format                                              */
/* Entrada: cadena, double, numero de digitos                           */
/* Salida : convierte un double a una cadena formato SCI                */
/************************************************************************/ 

void 
str_sci_format(gdouble       numericResult,
	       gint          formatSize,
	       gchar        *resultStr)
{
  gint    dec,sign, expSize = 0, expSign = 0;
  gchar  *tmpStr;
  
  if (formatSize >= MAX_FORMAT_SIZE)
    tmpStr = ecvt(numericResult,MAX_FORMAT_SIZE,&dec,&sign);
  else
    tmpStr = ecvt(numericResult,formatSize+1,&dec,&sign);
  
  if(dec > 0 ) 
    expSize = dec - 1;
  else {
    expSize = 1 - dec;
    expSign = 1;
  }
  dec = 1;  
  
  if(sign) 
    *resultStr++ = '-';
  
  *resultStr++ = *tmpStr++ ;
  *resultStr++ = '.';
  while( *tmpStr != '\0' )
    *resultStr++ = *tmpStr++ ;
  
  *resultStr++ = 'e';
  if(expSign)
    *resultStr++ = '-';
  else
    *resultStr++ = '+';
  gcvt(expSize,4,resultStr);
  
  return;
}


/************************************************************************/
/* Funcion: str_eng_format                                              */
/* Entrada: cadena, double, numero de digitos                           */
/* Salida : convierte un double a una cadena formato ENG                */
/************************************************************************/ 

void 
str_eng_format(gdouble       numericResult, 
	       gint          formatSize, 
	       gchar        *resultStr)
{
  gint    dec,sign, expSize = 0, expSign = 0;
  gchar  *tmpStr;
  
  if (formatSize >= MAX_FORMAT_SIZE)
    tmpStr = ecvt(numericResult,MAX_FORMAT_SIZE,&dec,&sign);
  else
    tmpStr = ecvt(numericResult,formatSize + 1,&dec,&sign);
  
  expSize = dec - 1;
  dec = 1;
  while( expSize % 3 != 0 ) {
    expSize--;
    dec++;
  }
  if(expSize < 0) {
    expSize = -expSize;
    expSign = 1; 
  }
  
  if(formatSize + 1 < dec)
    *(tmpStr + ++formatSize) = '0';
  
  if(sign) 
    *resultStr++ = '-';
  
  while( *tmpStr != '\0' ) {
    if( dec == 0 ) 
      *resultStr++ = '.';
    *resultStr++ = *tmpStr++;
    dec--;
  }
  
  *resultStr++ = 'e';
  if(expSign)
    *resultStr++ = '-';
  else
    *resultStr++ = '+';
  
  gcvt(expSize,4,resultStr);  
  
  return;
}


